--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

local dewdrop = AceLibrary("Dewdrop-2.0")
local quixote = AceLibrary("Quixote-1.0")
local crayon = AceLibrary("Crayon-2.0")
local tablet = AceLibrary("Tablet-2.0")

function nQuestLog:UpdateEvents()
	-- Zone change events
	if (self.db.profile.showQuestHighlight and not self.ZONEHOOKED) then
		self:RegisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
		self.ZONEHOOKED = true
		self:OnZoneChange()
	elseif(not self.db.profile.showQuestHighlight and self.ZONEHOOKED) then
		self:UnregisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
		self.ZONEHOOKED = false
	end
	if (not self.db.profile.showQuestHighlight) then
		self.highlightList = {}
	end
	
	-- Tooltip events:
	if (self.db.profile.showToolTip and not self.TOOLTIPHOOKED) then
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT","OnMouseoverUnit")
		self:HookScript(GameTooltip, "OnTooltipSetItem")
		self.TOOLTIPHOOKED = true
	elseif (not self.db.profile.showToolTip and self.TOOLTIPHOOKED) then
		self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT","OnMouseoverUnit")
		self:Unhook(GameTooltip, "OnTooltipSetItem")
		self.TOOLTIPHOOKED = false
	end
end

-------------------------------------------------- Zone Events : ------------------------------------------------

function nQuestLog:OnZoneChange(quick)
	self.highlightList = {}
	if (not self.db.profile.showQuestHighlight) then
		return
	end
	local strZoneText = string.lower(GetZoneText())
	local strSubZoneText = string.lower(GetSubZoneText())

	for _,iQuestId in quixote:QuestsByLevel() do
		local strTitle, _, _, strObjective, _, _, strZone = quixote:GetQuestById(iQuestId)
		-- Check for zone text in short quest objective description
		if strSubZoneText ~= "" and string.find(string.lower(strObjective),strSubZoneText,1,true) then
			self.highlightList[strTitle] = true
		end
		
		-- Now check the quest description against the main zone, maybe we got out of zone quests.
		if strZoneText ~= "" and string.lower(strZone) ~= strZoneText and string.find(string.lower(strObjective),strZoneText,1,true) then
			self.highlightList[strTitle] = true
		end
	end
	
	if (not quick) then
		self:Refresh()
	end
end

-------------------------------------------------- UI Events : --------------------------------------------------

function nQuestLog:OnTooltipUpdate()
	local cat = tablet:AddCategory()
	
	local strStatus = "Visible"
	if (not nQuestLogFrame:IsVisible()) then
		strStatus = "Hidden"
		if (self.db.char["visible"]) then
			strStatus = "Hidden (no quests to track)"
		end
	end
	
	cat:AddLine("text","Tracker " .. strStatus)
	
	tablet:SetHint("\n|cffeda55fClick|r to toggle Quest Tracker\n|cffeda55fRight Click|r for options")
end

function nQuestLog:OnClick(button)
	-- FuBar OnClick
	if (nQuestLogFrame:IsVisible()) then
		self:ToggleVisible(false)
	else
		self:ToggleVisible(true)
	end
end

function nQuestLog:OnTextUpdate()
	local iQuests = quixote:GetNumQuests()
	local color = crayon:GetThresholdHexColor((MAX_QUESTS - iQuests) / MAX_QUESTS)
	self:SetText(crayon:Colorize(color, iQuests))
end

function nQuestLog:OnMenuRequest()
	self:RefreshMenu()
	dewdrop:FeedAceOptionsTable(self.mainMenu)
end

function nQuestLog:OnQuestLogShiftClick(button,questid)
	local title,level,_,_,_,objectives = quixote:GetQuestById(questid)
	if (title == nil) then
		return
	end
	
	-- Link quest to chat
	local text = "[" .. level .. "]" .. title .. " "
	local totalGot,totalNeeded = 0,0
	
	if (button == "RightButton" and objectives > 0) then
		text = text .. "("
	end
	
	for i=1,objectives do
		local desc, _, got, needed = quixote:GetQuestStatusById(questid, i)
		totalGot = totalGot + got
		totalNeeded = totalNeeded + needed
		if (button == "RightButton") then
			-- Link quest objectives
			if (i > 1) then
				text = text .. ", "
			end
			text = text .. desc .. " " .. got .. "/" .. needed
		end
	end
	
	if (button == "RightButton" and objectives > 0) then
		text = text .. ") "
	end

	
	if (IsControlKeyDown()) then
		-- Link total objectives
		text = text .. totalGot .. "/" .. totalNeeded .. " "
	end
			
	if (ChatFrameEditBox:IsVisible()) then
		ChatFrameEditBox:Insert(text)
	else
		local chatType = DEFAULT_CHAT_FRAME.editBox:GetAttribute("chatType")
		SendChatMessage(text, chatType)
	end
end

function nQuestLog:OnQuestLogClickZone(button,zone)
	if (button == "LeftButton") then
		-- Fold/Unfold zone
		if self.db.char.hiddenZones[zone] then
			self.db.char.hiddenZones[zone] = nil
		else
			self.db.char.hiddenZones[zone] = true
		end
		self:Refresh()
	elseif (button == "RightButton") then
		-- Show popup menu for this zone
		local zoneVisibility = "Hide Zone"
		if self.db.char.hiddenZones[zone] then
			zoneVisibility = "Show Zone"
		end
		zoneVisibility = self:Colorize(zoneVisibility,"zoneHeader")
		
		local popupMenu = {
			type='group',
			args = {
		         showHide = {
		            type = 'execute',
		            name = zoneVisibility,
		            desc = 'Toggle zone visibility',
		            order = 1,
		            func = function()
		            	if (self.db.char.hiddenZones[zone]) then
		            		self.db.char.hiddenZones[zone] = nil
		            	else
		            		self.db.char.hiddenZones[zone] = true
		            	end
		            	self:Refresh()
		            	dewdrop:Close()
		            end
		        },
			}
		}
		
		for _,id in quixote:QuestsByZone(zone) do
			local title, level, tag, objective, complete, iObjectives = quixote:GetQuestById(id)
			local _, got, needed = self:Format_QuestObjectives(title,iObjectives,objective,id)
			local formatedTitle = self:Format_QuestTitle(title,level,tag,got,needed,complete)
			
			popupMenu.args[title] = {
				order = id + 5,
				name = formatedTitle,
				type = "toggle",
				desc = "Toggle showing " .. title,
				get = function()
					return self.db.char.watchedQuests[title]
				end,
				set = function(v)
					self.db.char.watchedQuests[title] = nil
					if (v) then
						self.db.char.watchedQuests[title] = true
					end
					self:Refresh()
				end
			}
		end
		
		dewdrop:Open( nQuestLogFrame,
			'children', function() dewdrop:FeedAceOptionsTable(popupMenu ) end,
			'cursorX', true, 'cursorY', true )
	end
end

function nQuestLog:OnQuestLogClickQuest(button,questid,zone)
	local title,level,_,_,_,objectives = quixote:GetQuestById(questid)
	if (title == nil) then
		return
	end


	if (button == "LeftButton") then
		if (IsAltKeyDown()) then
			-- Toggle objective visibility
			if self:ShouldShowObjective(title) then
				self.db.char.watchedObjectives[title] = false
			else
				self.db.char.watchedObjectives[title] = true
			end
			self:Refresh()
		else
			-- Show details frame
			if self:IsModuleEnabled("DetailsFrame") then
				self:GetModule("DetailsFrame"):ShowLog(questid)
			else
				local wasHidden = not QuestLogFrame:IsVisible()
				QuestLogFrame:Hide()
				
				if (GetQuestLogSelection() ~= questid or wasHidden) then
					SelectQuestLogEntry(questid)
					QuestLogFrame:Show()
				end
			end
		end
	end
end

-------------------------------------------------- Tooltip Events : --------------------------------------------------

function nQuestLog:OnMouseoverUnit()
	if quixote:IsQuestMob(UnitName('mouseover')) then
		for _,q,i,n in quixote:IterateQuestsForMob(UnitName('mouseover')) do
			GameTooltip:AddDoubleLine(q, i..'/'..n, 1, 1, 1, crayon:GetThresholdColor(i/n))
		end
		GameTooltip:Show()
	end
end

function nQuestLog:OnTooltipSetItem(tooltip, ...)
	local strName,iGot,iNeeded = quixote:IsQuestItem(tooltip:GetItem())
	if (strName) then
		tooltip:AddDoubleLine(strName, iGot..'/'..iNeeded, 1, 1, 1, crayon:GetThresholdColor(iGot/iNeeded))
		tooltip:Show()
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...)
end



-------------------------------------------------- Quest Events: --------------------------------------------------

function nQuestLog:OnQuestLog_Ready()
	self.refreshHold = nil
	self:OnZoneChange(true)
	if self.db.char["visible"] then
		self:ToggleVisible(true)
	end
	self:UpdateText()
end

function nQuestLog:OnQuest_Gained(strTitle)
	if (self.db.profile.trackNewQuests) then
		self.db.char.watchedQuests[strTitle] = true
	end
	self:UpdateText()
	
	self:OnZoneChange(true)
end

function nQuestLog:OnQuest_Lost(strTitle)
	self.db.char.watchedQuests[strTitle] = nil
	self.db.char.watchedObjectives[strTitle] = nil
	self:UpdateText()
end

function nQuestLog:OnQuest_Leaderboard_Update(title, _, _, desc)
	if (self.db.profile.trackWhenLooting and not self.db.char.watchedQuests[title]) then
		self.db.char.watchedQuests[title] = true
		self:Print("Found " .. desc .. " for " .. title .. " adding to quest tracker.")
	end
end
