local moduleName = "BlizzardFrames"
local BlizzardFrames = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1")

local _G = getfenv(0)
local quixote = AceLibrary("Quixote-1.0")

local L = AceLibrary("AceLocale-2.2"):new(moduleName)
L:RegisterTranslations("enUS", function() return {
    ["Description"] = "This module hooks blizzard frames and displays quest levels",
} end)


function BlizzardFrames:OnModuleLoad()
	self.description = L["Description"]

	self.core:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
	})
end

function BlizzardFrames:ToggleEnabled(v)
	self.db.profile.enabled = v
	if (v) then
		self:RegisterEvent("GOSSIP_SHOW", "OnGossipShow")
		self:RegisterEvent("QUEST_GREETING", "OnQuestGreeting")
		self:SecureHook("QuestLog_Update")
	else
		self:UnregisterEvent("GOSSIP_SHOW", "OnGossipShow")
		self:UnregisterEvent("QUEST_GREETING", "OnQuestGreeting")
		self:Unhook("QuestLog_Update")
	end
end

--This is pretty much how Minimalist does it, which it credits to AutoSelect.
function BlizzardFrames:OnGossipShow()
	local buttonindex = 1
	local list, button
	if GetGossipAvailableQuests() then
		list = {GetGossipAvailableQuests()}
		for i = 2,getn(list),2 do
			button = _G["GossipTitleButton"..buttonindex]
			button:SetText(format('[%d] %s',list[i],list[i-1]))
			buttonindex = buttonindex + 1
		end
		buttonindex = buttonindex + 1
		for k in pairs(list) do
			list[k] = nil
		end
	end
	if GetGossipActiveQuests() then
		list = {GetGossipActiveQuests()}
		for i = 2,getn(list),2 do
			button = _G["GossipTitleButton"..buttonindex]
			button:SetText(format('[%d] %s',list[i],list[i-1]))
			buttonindex = buttonindex + 1
		end
		for k in pairs(list) do
			list[k] = nil
		end
	end
end

--This is pretty much how Minimalist does it, which it credits to AutoSelect.
function BlizzardFrames:OnQuestGreeting()
	local numactive,numavailable = GetNumActiveQuests(), GetNumAvailableQuests()
	local title, level, button
	local o,GetTitle,GetLevel = 0,GetActiveTitle,GetActiveLevel
	for i=1, numactive + numavailable do
		if i == numactive + 1 then
			o,GetTitle,GetLevel = numactive,GetAvailableTitle,GetAvailableLevel
		end
		title,level = GetTitle(i-o), GetLevel(i-o)
		button = _G["QuestTitleButton"..i]
		button:SetText(format('[%d] %s',level,title))
	end
end

-- Copied from QuestsFu
function BlizzardFrames:QuestLog_Update()
	--This is fragile, but lets us ignore event sequencing.  It'll break if anyone is meddling with the output of GetQuestLogTitle.
	--(questIndexes can change on every QUEST_LOG_UPDATE; this *will* be called before Quixote updates from that event, because Quixote is
	--throttling to avoid crashing the client on edge cases. I blame Blizzard for it being possible to crash the game by scanning the quest log. So.)
	for i = 1, QUESTS_DISPLAYED, 1 do
		--local questIndex = i + FauxScrollFrame_GetOffset(QuestLogListScrollFrame)
		local titleLine = _G["QuestLogTitle"..i]
		if titleLine and titleLine:GetText() then
			local title, level, tag = quixote:GetQuest(titleLine:GetText():trim())
			if title then
				titleLine:SetText("[" .. level .. tag .. "] " .. title)
				local check = _G["QuestLogTitle" .. i .. "Check"]
				if check:IsVisible() then
					check:SetPoint("LEFT", titleLine, "LEFT", QuestLogDummyText:GetWidth() + 24, 0)
				end
			end
		end
	end
end
