--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

-- Todo, more notification options, play sound, support notify through MikSbt/SCT

local moduleName = "Notifications"
local Notification = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1", "AceConsole-2.0")
local quixote = AceLibrary("Quixote-1.0")

local L = AceLibrary("AceLocale-2.2"):new(moduleName)
L:RegisterTranslations("enUS", function() return {
    ["Description"] = "Notifies you when something is completed",
   	["%s in %s (%d,%d)"] = true,
   	["%s in %s (%d,%d) (%d yd)"] = true,
   	["Notify on"] = true,
   	["Finished objective"] = true,
   	["Finished quest"] = true,
   	["Getting quest item"] = true,
   	["completed"] = true,
   	["only when quest is hidden"] = true,
   	["when the quest is hidden"] = true,
} end)

function Notification:OnModuleLoad()
	-- We'd want to set self.name and self.description here? (used for the menu made by parent atleast)
	self.description = L["Description"]
	
	self.core:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
		["notifyFinishedQuest"] = true,
		["notifyFinishedObjective"] = false,
		["notifyLootedQuestItem"] = true,
		["notifyLootedHiddenQuestItemOnly"] = true,
	})
end

function Notification:ToggleEnabled(v)
	self.db.profile.enabled = v
	if (v) then
		self:RegisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
	else
		self:UnregisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
	end
end

function Notification:GetMenu(oldMenu)
	local menu = {
		type="group",
		name=moduleName,
		order=oldMenu.order,
		desc=self.description,
		args = {
			toggleEnabled = oldMenu,
			notifyFinishedObjective = {
				type="toggle",
				name=L["Finished objective"],
				desc=L["Notify on"] .. " " .. L["Finished objective"],
				order=1300,
				get=function()
					return self.db.profile.notifyFinishedObjective
				end,
				set=function(v)
					self.db.profile.notifyFinishedObjective = v
				end
			},
			notifyFinishedQuest = {
				type="toggle",
				name=L["Finished quest"],
				desc=L["Notify on"] .. " " .. L["Finished quest"],
				order=1301,
				get=function()
					return self.db.profile.notifyFinishedQuest
				end,
				set=function(v)
					self.db.profile.notifyFinishedQuest = v
				end
			},
			notifyFinishedQuestItem = {
				type="toggle",
				name=L["Getting quest item"],
				desc=L["Notify on"] .. " " .. L["Getting quest item"] .. " " .. L["completed"],
				order=1302,
				disabled=function()
					return self.db.profile.notifyLootedHiddenQuestItemOnly
				end,
				get=function()
					return self.db.profile.notifyLootedQuestItem
				end,
				set=function(v)
					self.db.profile.notifyLootedQuestItem = v
				end
			},
			notifyFinishedHiddenQuestItem = {
				type="toggle",
				name=" - " .. L["only when quest is hidden"],
				desc=L["Notify on"] .. " " .. L["Getting quest item"] .. " " .. L["when the quest is hidden"],
				order=1303,
				disabled=function()
					return not self.db.profile.notifyLootedQuestItem
				end,
				get=function()
					return self.db.profile.notifyLootedHiddenQuestItemOnly
				end,
				set=function(v)
					self.db.profile.notifyLootedHiddenQuestItemOnly = v
				end
			},
		}
	}
	
	return menu	
end

function Notification:OnQuest_Leaderboard_Update(title, _, _, desc, had, got, needed)
	if (self.db.profile.notifyFinishedQuest) then
		local _, _, _, _, complete = quixote:GetQuestByName(title)
		if complete == 1 then
			self:Print(L["Finished quest"] .. " " .. title)
			return
		end
	end
	
	if (got == needed and self.db.profile.notifyFinishedObjective) then
		self:Print(L["Finished objective"] .. " " .. desc)
	else
		local _, _, _, _, _, _, zone = quixote:GetQuestByName(title)
		if (self.db.profile.notifyLootedQuestItem and not self.db.profile.notifyLootedHiddenQuestItemOnly) then
			-- Show alert, not locked to hidden only
			self:Print(desc .. " " .. got .. "/"  .. needed)
		elseif ((self.db.profile.notifyLootedHiddenQuestItemOnly) and (not self.core.db.char.watchedQuests[title] or self.core.db.char.hiddenZones[zone])) then
			-- Show is hidden only, but quest or zone is hidden.
			self:Print(desc .. " " .. got .. "/"  .. needed)
		end
	end
end
